;#############################################################################
;
; Usage example for the procedure PPXF, which
; implements the Penalized Pixel-Fitting (pPXF) method by
; Cappellari M., & Emsellem E., 2004, PASP, 116, 138.
; The example also shows how to include a library of templates
; and how to mask gas emission lines if present.
; 
; MODIFICATION HISTORY:
;   V1.0: Written by Michele Cappellari, Leiden 11 November 2003
;   V1.1: Log rebin the galaxy spectrum. Show how to correct the velocity
;       for the difference in starting wavelength of galaxy and templates.
;       MC, Vicenza, 28 December 2004
;   V1.11: Included explanation of correction for instrumental resolution.
;       After feedback from David Valls-Gabaud. MC, Venezia, 27 June 2005
;   V2.0: Included example routine to determine the goodPixels vector
;       by masking known gas emission lines. MC, Oxford, 30 October 2008
;   V2.01: Included instructions for high-redshift usage. Thanks to Paul Westoby
;       for useful feedback on this issue. MC, Oxford, 27 November 2008
;   V2.02: Included example for obtaining the best-fitting redshift.
;       MC, Oxford, 14 April 2009
;       
;#############################################################################
function determine_goodPixels, logLam, lamRangeTemp, redshift
;
; DETERMINE_GOODPIXELS: Example routine to generate the vector of goodPixels 
;     to be used as input keyword for the routine PPXF. This is useful to mask 
;     gas emission lines. But it has to be adapted to the wavelength region
;     of interest and to the presence of gas emission lines.
; 
; INPUT PARAMETERS:
; - LOGLAM: Natural logarithm of the wavelength in Angstrom of each pixel 
;     of the log rebinned *galaxy* spectrum.
; - LAMRANGETEMP: Two elements vectors [lamMin2,lamMax2] with the minimum and
;     maximum wavelength in Angstrom in the stellar *template* used in PPXF.
; - REDSHIFT: Estimate of the galaxy redshift in km/s.
; 
; Michele Cappellari, Leiden, 9 September 2005

lines = [4861.3d, 4958.9d, 5006.9d] ; Wavelengths of Hbeta and [OIII] doublet
dv = [800d, 500d, 800d] ; width/2 of masked gas emission region in km/s
c = 299792.458d ; speed of light in km/s
flag = bytarr(n_elements(logLam))
for j=0,n_elements(lines)-1 do $
    flag or= logLam gt alog(lines[j]) + (redshift - dv[j])/c $
         and logLam lt alog(lines[j]) + (redshift + dv[j])/c
flag or= logLam lt alog(lamRangeTemp[0]) + (redshift + 900d)/c ; Mask edges of
flag or= logLam gt alog(lamRangeTemp[1]) + (redshift - 900d)/c ; stellar library

return, where(flag eq 0)
end
;------------------------------------------------------------------------------
pro ppxf_example

loadct, 12 ; Green=40, Blue=100, Magenta=130, Red=200

; Read a SAURON galaxy spectrum and rebin it logarithmically.
; As the keyword VELSCALE has not been previously defined, LOG_REBIN will
; keep the same number of pixels in the spectrum, and will provide in output
; the velocity scale in km/s, to be used in the subsequent calls below.
;
fits_read, 'spectra/galaxy.fits', gal_lin, h
lamRange1 = sxpar(h,'CRVAL1') + [0d,sxpar(h,'CDELT1')*(sxpar(h,'NAXIS1')-1d)]

; If the galaxy is at a significant redshift (z > 0.03), one would need to apply 
; a large velocity shift in PPXF to match the template to the galaxy spectrum.
; This would require a large initial value for the velocity (V > 1e4 km/s) 
; in the input parameter START = [V,sig]. This can cause PPXF to stop! 
; The solution consists of bringing the galaxy spectrum roughly to the 
; rest-frame wavelength, before calling PPXF. In practice there is no 
; need to modify the spectrum before the usual log_rebin, as a red shift
; corresponds to a linear shift of the log-rebinned spectrum. 
; One just needs to compute the wavelength range in the rest-frame. 
; This is done with the following two commented lines:
;
; z = 1.23 ; Initial estimate of the galaxy redshift
; lamRange1 = lamRange1/(1+z) ; Compute approximate restframe wavelength range

log_rebin, lamRange1, gal_lin, galaxy, logLam1, VELSCALE=velScale
error = galaxy*0 + 1              ; Assume constant errors per pixel here

; Read the list of filenames from the Single Stellar Population library
; by Vazdekis (1999, ApJ, 513, 224) http://www.iac.es/galeria/vazdekis/.
; A subset of the library is included for this example with permission.
;
vazdekis = file_search('spectra/Rbi1.30z*.fits',COUNT=nfiles)

; Extract the wavelength range and logarithmically rebin one spectrum
; to the same velocity scale of the SAURON galaxy spectrum, to determine
; the size needed for the array which will contain the template spectra.
;
fits_read, vazdekis[0], ssp, h
lamRange2 = sxpar(h,'CRVAL1') + [0d,sxpar(h,'CDELT1')*(sxpar(h,'NAXIS1')-1d)]
log_rebin, lamRange2, ssp, sspNew, logLam2, VELSCALE=velScale
templates = dblarr(n_elements(sspNew),nfiles)

; Logarithmically rebin the whole Vazdekis library of spectra,
; and store each template as a column in the array TEMPLATES,
; after convolving it with the quadratic difference between
; the SAURON and the Vazdekis instrumental resolution.
;
; Example: Vazdekis spectra have a resolution of 1.8A FWHM at 5100A
; this corresponds to sigma ~ 1.8/5100*3e5/2.355 = 45 km/s.
; SAURON has an instrumental resolution of sigma = 108 km/s.
; The quadratic difference is sigma = sqrt(108^2 - 45^2) = 98 km/s
; (the above reasoning can be applied if the shape of the instrumental
; spectral profiles can be well approximated by a Gaussian).
;
sigma = 98d/velScale ; Quadratic sigma difference in pixels Vazdekis --> SAURON
lsf = psf_Gaussian(NPIXEL=8*sigma, ST_DEV=sigma, /NORM, NDIM=1)
for j=0,nfiles-1 do begin
    fits_read, vazdekis[j], ssp
    log_rebin, lamRange2, ssp, sspNew, VELSCALE=velScale
    templates[*,j] = convol(sspNew,lsf)/median(sspNew) ; Degrade template to SAURON resolution
endfor

; The galaxy and the template spectra do not have the same starting wavelength.
; For this reason an extra velocity shift DV has to be applied to the template
; to fit the galaxy spectrum. We remove this artificial shift by using the
; keyword VSYST in the call to PPXF below, so that all velocities are
; measured with respect to DV. This assume the redshift is negligible.
; In the case of a high-redshift galaxy one should de-redshift its 
; wavelength to the rest frame before using the line below (see above).
;
c = 299792.458d
dv = (logLam2[0]-logLam1[0])*c ; km/s

redshift = 450d ; Initial estimate of the galaxy redshift in km/s
goodPixels = determine_goodPixels(logLam1,lamRange2,redshift)

; Uncomment the following line to set goodPixels by hand, instead 
; of using the optional determine_goodPixels routine above.
;
;goodPixels = [range(1,35),range(50,125),range(145,165),range(190,414)]

; Here the actual fit starts. The best fit is plotted on the screen.
; Gas emission lines are excluded from the pPXF fit using the GOODPIXELS keyword.
;
start = [redshift, 180d] ; (km/s), starting guess for [V,sigma]
ppxf, templates, galaxy, error, velScale, start, sol, $
    GOODPIXELS=goodPixels, /PLOT, MOMENTS=4, DEGREE=4, VSYST=dv

; If the galaxy is at significant redshift z and the wavelength has been
; de-redshifted with the two lines near the beginning of this procedure,
; the best-fitting redshift is now given by the following commented line:
;    
;print, 'Best-fitting redshift z:', (z + 1)*(1 + sol[0]/c) - 1        

end
;------------------------------------------------------------------------------
